#include <avr/io.h>
#include <util/delay.h>
#include <stdlib.h>
#include <stdbool.h>

#include "display.h"

void display_tick()
{
	PORTD |= DISPLAY_E;
	_delay_ms(10);
	PORTD &= ~DISPLAY_E;	
}

void display_num(int num, char *buff)
{
	itoa(num, buff, 10);
	display_text(buff);	
}

void display_clock(char hh, char mm, char ss, char *clkbuf)
{	
	if (hh < 10)
		display_char('0');
	display_num(hh, clkbuf);

	display_char(':');

	if (mm < 10)
		display_char('0');
	display_num(mm, clkbuf);

	display_char(':');

	if (ss < 10)
		display_char('0');
	display_num(ss, clkbuf);
}

void display_char(char chr) 
{	
	PORTD |= DISPLAY_RS;
	PORTB = chr;
	display_tick();
}

void display_text(char *str) 
{
	while (*str) {
		display_char(*str);
		str++;
	} 
}

void display_goto(int row, int col)
{
	PORTD &= ~DISPLAY_RS;
	PORTD &= ~DISPLAY_RW;	
	if (row == ROW_TWO)
		PORTB = 64+col;
	else
		PORTB = col;
		
	PORTB |= (1 << PD7);
	display_tick();
}

void display_busywait()
{
	/* RS=0, R/W=1, PORT D Input */
	DDRB = 0;
	PORTD &= ~DISPLAY_RS;
	PORTD |= DISPLAY_RW;
	
	while ((PORTB & _BV(PD7)) != 0);
		
	/* Set PORT D to Output again */
	DDRB = 0xFF;
}

void display_shift(bool display, bool left)
{
	PORTD &= ~DISPLAY_RS;
	PORTD &= ~DISPLAY_RW;
	
	PORTB = 0;
	if (display) {
		if (left)			
			PORTB |= (1 << 4) | (1 << 3);
		else
			PORTB |= (1 << 4) | (1 << 3) | (1 << 2);			
	} else {
		if (left)			
			PORTB |= (1 << 4);
		else
			PORTB |= (1 << 4) | (1 << 2);			
	}

	display_tick();
}

void display_clear()
{
	/* Clear display */
	PORTD &= ~DISPLAY_RS;
	PORTD &= ~DISPLAY_RW;	
	PORTB = 0x1;
	display_tick();	
}

void display_init()
{	
	_delay_ms(20);

	display_tick();
	// Set I/O port directions
	DDRD = 0xFF;
	DDRB = 0xFF;

	/* Display clear */
	PORTB = 0x1;
	display_tick();

	/* Function set (8-bit interface) */
	PORTB = 0x38;
	display_tick();
	_delay_ms(2);

	/* Display ON */
	// PORTB = 0x0F; /* for blink + cursor */
	PORTB = 0x0C;
	display_tick();
	_delay_ms(2);

	/* Entry mode set */
	PORTB = 0x06;
	display_tick();
	_delay_ms(2);

	/* Display clear */
	PORTB = 0x1;
	display_tick();
}
