#include <stdlib.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <avr/io.h> 
#include <limits.h>
#include <avr/sleep.h>

#include "menu.h"
#include "temperature.h"
#include "display.h"

char hours = 23, minutes = 59, seconds;
int keystate;
int ctemp, maxtemp = INT_MIN, mintemp = INT_MAX;
int chum, maxhum = INT_MIN, minhum = INT_MAX;

ISR(TIMER1_OVF_vect)
{
	TCNT1H=0xC2;
	TCNT1L=0xF7;           	

	seconds++;
	if (seconds == 60) {
		seconds = 0;		

		minutes++;
		if (minutes == 60) {
			minutes = 0;			

			hours++;
			if (hours == 24)
				hours = 0;
		}
	}		

	if (seconds % 10 == 0)
		temp_sample();	

	menu_draw();
}

ISR(ADC_vect)
{	
	ctemp = temp_value();
	if (ctemp > maxtemp)
		maxtemp = ctemp;

	if (ctemp < mintemp)
		mintemp = ctemp;	
	
	menu_draw();		
}

/* "Nedre knappen" */
ISR(INT0_vect)
{
	menu_keypress(BUTTON_RESET);
	_delay_loop_2(65000);
}

/* "vre knappen" */
ISR(INT1_vect)
{
	menu_keypress(BUTTON_MODE);
	_delay_loop_2(65000);
}

void init()
{
	/* Timer setup, interrupts at 1 Hz */
	TCCR1A = 0;
	TCCR1B = _BV(CS11) | _BV(CS10); 	
	TCNT1H = 0xC2;
	TCNT1L = 0xF7;           	
	TIMSK = _BV(TOIE1);

	/* External interrupt setup */
	MCUCR = _BV(ISC00) | _BV(ISC01) | _BV(ISC10) | _BV(ISC11); 
	GICR = _BV(INT0) | _BV(INT1);   	
			
	/* Power management */
	// set_sleep_mode(SLEEP_MODE_IDLE);
	// sleep_enable();

	temp_init();
	menu_init();	

	/* Enable all interrupts */
	sei();	
	
	/* Initial temperature sample */
	temp_sample();	
}

int main(void)
{		
	init();
		
	for(;;) { 				
		// sleep_cpu();
	}

    return 0;
}
